import os

from machine import Pin, SoftI2C
import machine
import time
from umqtt.simple import MQTTClient
import json
import gc

cycle_time = 5

# SHT31 default I2C address
SHT31_I2C_ADDRESS = 0x44

# MQTT configuration
mqtt_server = os.environ['MQTT_SERVER']
mqtt_port = 30083
mqtt_topic = "sensor/temperature_humidity"

i2c = SoftI2C(scl=Pin(22), sda=Pin(21), freq=100000)  # ESP32


def initialize_sht31():
    try:
        # Send the command to initialize the sensor
        i2c.writeto(SHT31_I2C_ADDRESS, bytes([0x30, 0xA2]))

        # Wait for the sensor to initialize (datasheet says at least 10ms)
        time.sleep_ms(20)

        return True
    except Exception as e:
        print("Error initializing sensor:", e)
        return False


def read_sht31():
    try:
        i2c.writeto(SHT31_I2C_ADDRESS, bytes([0x2C, 0x06]))

        time.sleep_ms(20)

        # Read 6 bytes of data (2 bytes temperature, 2 bytes humidity, 2 bytes CRC)
        data = i2c.readfrom(SHT31_I2C_ADDRESS, 6)

        temperature_celsius = -45 + (175 * ((data[0] << 8) + data[1]) / 65535.0)
        temperature_fahrenheit = (temperature_celsius * 9 / 5) + 32

        humidity = 100 * ((data[3] << 8) + data[4]) / 65535.0

        return temperature_fahrenheit, humidity
    except Exception as e:
        print("Error reading sensor:", e)
        return None, None


def main():
    gc.enable()

    last_temperature = None
    last_humidity = None

    while True:
        # gc.collect()
        if initialize_sht31():
            # Maybe add try/except here?
            temperature, humidity = read_sht31()
            if temperature is not None and humidity is not None:
                if (last_temperature is None or abs(temperature - last_temperature) <= 3) and \
                   (last_humidity is None or abs(humidity - last_humidity) <= 3):
                    last_temperature = temperature
                    last_humidity = humidity
                    uptime_seconds = time.time()
                    allocated_mem = gc.mem_alloc()
                    payload = {
                        "temperature": temperature,
                        "humidity": humidity,
                        "seconds_up": uptime_seconds,
                        "allocated_mem": allocated_mem
                    }

                    try:
                        client = MQTTClient("hostname", mqtt_server, keepalive=60)
                        client.connect()
                        client.publish(mqtt_topic, json.dumps(payload))
                        client.disconnect()
                        del client
                        print("Published to MQTT: Temperature: {:.2f} F, Humidity: {:.2f} %".format(temperature, humidity))

                    except Exception as e:
                        print("Error sending MQTT message:", e)

                else:
                    print("Discarded data: Temperature or humidity differs by more than 3 from the most recent value.")
                    machine.reset()

            else:
                print(f'Failed to read sensor. Retrying in {cycle_time} seconds.')

            time.sleep(cycle_time)


if __name__ == "__main__":
    main()



from pathlib import Path
import click


def validate_serial_port(ctx, param, value):
    if not value:
        raise click.BadParameter('Serial port is required as first argument.')
    return value


@click.command()
@click.argument('serial-port', callback=validate_serial_port)
@click.option('--baud-rate', default=115200, help='Baud rate for the serial communication (default: 115200)')
@click.option('--scripts-path', help='Absolute path to the directory that holds the scripts to flash.', default="./scripts")
@click.option('--chip', required=True, type=click.Choice(['ESP32', 'ESP8266']), help='Specify the chip type (ESP32 or ESP8266)')
@click.option('--new-firmware', is_flag=True, default=False, help='Erase and flash firmware. If --firmware-file is not specified, this script will attempt to download the latest firmware from micropython.org')
@click.option('--firmware-file', default=None, help='Path to Micropython firmware bin to flash')
@click.option('--write-env-vars', is_flag=True, default=False, help='Overwrite environment variables in the script (os.environ[\'ENV_VAR\']) with local environment variables')
def main(serial_port, baud_rate, scripts_path, chip, new_firmware, firmware_file, write_env_vars):
    from esp_helper import firmware
    from esp_helper import scripts

    if new_firmware:
        firmware.erase_and_flash_firmware(chip, serial_port, firmware_file, baud_rate)

    scripts_directory = Path(scripts_path)

    if not scripts_directory.exists() or not scripts_directory.is_dir():
        raise FileNotFoundError(f"The directory '{scripts_directory}' does not exist or is not a directory.")

    if write_env_vars:
        scripts.substitute_and_flash(scripts_directory, serial_port, baud_rate)

    else:
        scripts.flash_to_esp(scripts_directory, serial_port, baud_rate)


if __name__ == "__main__":
    main()



import requests
from bs4 import BeautifulSoup
import subprocess
import urllib.request
import os


def erase_firmware(chip, port, baud_rate):
    try:
        # Construct the command
        command = f"esptool --port {port} --baud {baud_rate} --chip {chip} erase_flash"

        # Run the command using subprocess
        subprocess.run(command, shell=True, check=True)

        print("Firmware erased successfully.")

    except subprocess.CalledProcessError as e:
        print(f"Error occurred while erasing firmware: {e}")


def flash_firmware(chip, port, firmware_file, baud_rate):
    if chip == "ESP32":
        command = f"esptool --port {port} --baud {baud_rate} --chip {chip} write_flash --flash_size detect --flash_mode dio -z 0x1000 {firmware_file}"

    if chip == "ESP8266":
        command = f"esptool --port {port} --baud {baud_rate} --chip {chip} write_flash --flash_size=detect -fm dout 0 {firmware_file}"

    try:
        subprocess.run(command, shell=True, check=True)
        print("Firmware flashed successfully.")
    except subprocess.CalledProcessError as e:
        print(f"Error occurred while flashing firmware: {e}")


def get_latest_micropython_download_link(url):
    try:
        response = requests.get(url)
        response.raise_for_status()

        soup = BeautifulSoup(response.content, 'html.parser')

        links = soup.find_all('a')

        latest_version_link = None
        for link in links:
            href = link.get('href')
            if href and href.endswith('.bin'):
                if link.find_previous(string=lambda text: '(latest)' in text):
                    continue

                latest_version_link = f"https://micropython.org{href}"
                break

        if not latest_version_link:
            raise ValueError("No '.bin' file found on the page before '(latest)'")

        return latest_version_link

    except IndexError as e:
        print("Error: Index out of range. Make sure the website structure has not changed.")
        raise e
    except Exception as e:
        print(f"An error occurred: {e}")
        raise


def erase_and_flash_firmware(chip, port, firmware_file, baud_rate):
    if not firmware_file:
        if chip == "ESP32":
            download_page = "https://micropython.org/download/ESP32_GENERIC/"

        if chip == "ESP8266":
            download_page = "https://micropython.org/download/ESP8266_GENERIC/"

        firmware_bin_download_link = get_latest_micropython_download_link(download_page)
        urllib.request.urlretrieve(firmware_bin_download_link, "latest.bin")
        firmware_file = "latest.bin"
    erase_firmware(chip, port, baud_rate)
    flash_firmware(chip, port, firmware_file, baud_rate)
    if firmware_file == "latest.bin":
        os.remove("latest.bin")
    exit(0)



import os
import re
import shutil
from pathlib import Path
import time


def create_temp_dir(name):
    tmp_dir = Path.cwd() / name
    if tmp_dir.exists():
        shutil.rmtree(tmp_dir)
    tmp_dir.mkdir()
    return tmp_dir


def substitute_env_variables(file_path, tmp_dir):
    with open(file_path, 'r') as file:
        script_content = file.read()

    pattern = r"os\.environ\['(\w+)'\]"

    def replace_with_env_value(match):
        env_var = match.group(1)
        env_value = os.getenv(env_var)
        if env_value is None:
            raise ValueError(f"Environment variable '{env_var}' not found.")
        return repr(env_value)

    modified_script_content = re.sub(pattern, replace_with_env_value, script_content)

    original_filename = Path(file_path).name
    modified_file_path = tmp_dir / original_filename

    with open(modified_file_path, 'w') as modified_file:
        modified_file.write(modified_script_content)

    print(f"Modified script saved as {modified_file_path}")

    relative_path = modified_file_path.relative_to(Path.cwd())
    return relative_path


def flash_to_esp(src_dir, serial_port, baud_rate):
    for file_path in src_dir.glob('*'):
        file_path_str = str(file_path).replace('\\', '/')
        rshell_command = f'rshell -p {serial_port} --baud {baud_rate} cp "{file_path_str}" /pyboard'

        try:
            exit_code = os.system(rshell_command)
            if exit_code == 0:
                print(f"Successfully flashed {file_path_str} to ESP32.")
            else:
                print(f"Failed to flash {file_path_str} with exit code {exit_code}.")
        except Exception as e:
            print(f"An error occurred while flashing {file_path_str}: {e}")
        time.sleep(2)  # Flashing too fast seems to cause errors


def substitute_and_flash(scripts_directory, serial_port, baud_rate):
    tmp_dir = create_temp_dir("temp")

    try:
        for script_file in scripts_directory.glob('*.py'):
            substitute_env_variables(script_file, tmp_dir)

        relative_dir = tmp_dir.relative_to(Path.cwd())
        flash_to_esp(relative_dir, serial_port, baud_rate)
    finally:
        shutil.rmtree(tmp_dir)



from setuptools import setup, find_packages

setup(
    name='esp-helper',
    version='0.0.6',
    packages=find_packages(),
    install_requires=[
        'click==8.1.7',
        'requests==2.32.3',
        'beautifulsoup4==4.12.3',
        'pathlib==1.0.1',
        'urllib3==2.2.2',
    ],
    author='Jake Barbieur',
    author_email='barbieur@gmail.com',
    description='A helper application for ESP development',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    url='https://gitlab.com/jake-barbieur/esp-helper',
    entry_points={
        'console_scripts': [
            'esp-helper = esp_helper.main:main',
        ],
    },
)



import unittest
from unittest.mock import patch, Mock
from esp_helper.firmware import get_latest_micropython_download_link


class TestFirmware(unittest.TestCase):
    @patch('esp_helper.firmware.requests.get')
    def test_get_latest_micropython_download_link(self, mock_get):
        html_content = '''
                <strong>
                <a href="/resources/firmware/ESP32_GENERIC-20240602-v1.23.0.bin">v1.23.0 (2024-06-02) .bin</a>
                </strong>
                / <a href="/resources/firmware/ESP32_GENERIC-20240602-v1.23.0.app-bin">[.app-bin]</a>
                / <a href="/resources/firmware/ESP32_GENERIC-20240602-v1.23.0.elf">[.elf]</a>
                / <a href="/resources/firmware/ESP32_GENERIC-20240602-v1.23.0.map">[.map]</a>
                / <a href="https://github.com/micropython/micropython/releases/tag/v1.23.0">[Release notes]</a>
                (latest)
        '''

        mock_response = Mock()
        mock_response.status_code = 200
        mock_response.content = html_content
        mock_get.return_value = mock_response

        url = 'http://micropython.org'
        expected_link = 'https://micropython.org/resources/firmware/ESP32_GENERIC-20240602-v1.23.0.bin'
        result = get_latest_micropython_download_link(url)
        self.assertEqual(result, expected_link)

    @patch('esp_helper.firmware.requests.get')
    def test_get_latest_micropython_download_link_no_bin(self, mock_get):
        html_content = '''
        <html>
        <body>
            <a href="/resources/firmware/">Download Latest</a>
            <a href="/resources/firmware/">(latest)</a>
        </body>
        </html>
        '''
        mock_response = Mock()
        mock_response.status_code = 200
        mock_response.content = html_content
        mock_get.return_value = mock_response

        url = 'http://example.com'
        with self.assertRaises(ValueError) as context:
            get_latest_micropython_download_link(url)
        self.assertEqual(str(context.exception), "No '.bin' file found on the page before '(latest)'")

    def test_get_latest_micropython_download_link_live_esp32(self):
        url = 'https://micropython.org/download/ESP32_GENERIC/'
        result = get_latest_micropython_download_link(url)
        self.assertTrue(result.startswith('https://micropython.org/resources/firmware'))

    def test_get_latest_micropython_download_link_live_esp8266(self):
        url = 'https://micropython.org/download/ESP8266_GENERIC/'
        result = get_latest_micropython_download_link(url)
        self.assertTrue(result.startswith('https://micropython.org/resources/firmware'))


if __name__ == '__main__':
    unittest.main()
